@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009,2010 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -65,6 +65,10 @@ static int portflags    = 0;
 static int oldflags     = 0;
 static int rtscts       = 0;
 
+static int ivHeartbeat  = 0;
+static int ivInCommand  = 0;
+
+
 static struct termios oldterm;
 static struct termios portterm;
 
@@ -76,13 +80,14 @@ int bt_open_port(char* port)
 {
 	
 	/* get a file descriptor */
-	if ((portfd = open(port, O_RDWR | O_NOCTTY | O_NDELAY)) < 0) {
+	if ((portfd = open(port, O_RDWR|O_NOCTTY|O_NDELAY/*|O_CLOEXEC fails to compile ? */)) < 0) {
 		sprintf(tmp, "can not open %s", port);
 		logger("ERR", tmp);
 		printf("ERROR: can not open %s\n", port);
 		portfd = 0;
 		return -1;
 	}
+	fcntl(portfd,F_SETFD,FD_CLOEXEC);
 
 	/* check to see that the file is a terminal */
 	if (!isatty(portfd)) {
@@ -136,7 +141,7 @@ int bt_close_port(int final)
 	int retval = 0;
 
 	if (!portfd) { /* already closed */
-		logger("INF", "Already closed ?");
+		if (final) logger("INF", "Already closed ?");
 		return -1;
 	}
 
@@ -415,7 +420,7 @@ int unix_open_port(char* port)
 	
 	struct sockaddr_un serveraddr;
 
-	portfd = socket(AF_UNIX, SOCK_STREAM, 0);
+	portfd = socket(AF_UNIX, SOCK_STREAM|SOCK_CLOEXEC, 0);
 	if (portfd < 0) {
 		sprintf(tmp, "can not open UNIX %s", port);
 		logger("ERR", tmp);
@@ -445,7 +450,7 @@ int unix_close_port()
 	//printf("INFO: Close port\n");
 
 	if (!portfd) { /* already closed */
-		printf("INFO: Already closed ?\n");
+		//printf("INFO: Already closed ?\n");
 		return -1;
 	}
 
@@ -678,7 +683,7 @@ int openSocketPort(int type, int port, char *path)
 		return -1;
 	}
 
-	if ((sportfd = socket(addFamily, SOCK_STREAM, proto)) < 0) {
+	if ((sportfd = socket(addFamily, SOCK_STREAM|SOCK_CLOEXEC, proto)) < 0) {
 		logger("ERR", "opening socket");
 		printf("ERROR: opening socket\n");
 	
@@ -770,7 +775,7 @@ int openSocketPort(int type, int port, char *path)
 
 int closeSocketPort(int type, int final) 
 {
-	logger("INF", "closeSocketPort");
+	if (final) logger("INF", "closeSocketPort");
 	
 	if (portfd) { 
 		close(portfd);
@@ -889,13 +894,15 @@ int listenAndAcceptSocketConn(int type)
 int writeSocketConn(char* command, int count)
 {
 	logger("DBG", "writeSocketConn");
-
+	if (!command) return EXIT_OK;
+	
 	// send command
 	if (portfd > 0 && count > 0) {
         
 		memset(tmp, 0, MAXMAXLEN);
         	strcat(tmp, "writeSocketConn ");
                 
+		ivInCommand = 1;
                 int logSz = (count > 256 ? 255 : count);
                 
                 // it is possible to get binary data here
@@ -917,6 +924,28 @@ int writeSocketConn(char* command, int count)
 		logger("ERR", "error writing to socket: already closed");
 	}
 	return EXIT_NOK;
+}	
+
+static int writeByteSocketConn(int byte)
+{	
+	unsigned char byte2write[2];
+	byte2write[0] = (unsigned char) byte;
+	byte2write[1] = '\0';
+	
+	if (write(portfd, byte2write, 1) < 0) {
+		logger("ERR", "error writing byte to socket");
+		return EXIT_NOK;
+	}
+	return EXIT_OK;
+}
+
+static void writeIViewerHeartbeat()
+{	
+	// reply message is h=1\03
+	writeByteSocketConn(104); // h
+	writeByteSocketConn(61 ); // =
+	writeByteSocketConn(49 ); // 1
+	writeByteSocketConn(3  ); // \03
 }
 
 int writeBytesSocketConn(char* command) 
@@ -927,6 +956,8 @@ int writeBytesSocketConn(char* command)
 	if (portfd > 0 && command && command[0]) {
 		
 		char byteStr[MAXCKPDLEN];
+		memset(byteStr,0,MAXCKPDLEN);
+		
 		strncpy(byteStr,command,MAXCKPDLEN-1);
 
 		sprintf(tmp, "writeBytesSocketConn >%s<", byteStr);
@@ -953,15 +984,24 @@ int writeBytesSocketConn(char* command)
 
 			sprintf(tmp, "Next byte is >%s<", bStripped);
 			logger("DBG", tmp);
-		
-			unsigned char byte2write[2];
-			byte2write[0] = (unsigned char) atoi(bStripped);
-			byte2write[1] = '\0';
-		
-			if ( write(portfd, byte2write, 1) < 0) {
-				logger("ERR", "error writing to socket");
+			
+			if (writeByteSocketConn(atoi(bStripped)) != EXIT_OK) {
 				return EXIT_NOK;
 			}
+			
+			// send iViewer heartbeat message
+			if (strcmp(bStripped,"3") == 0) {
+				ivInCommand = 0;
+				
+				if (ivHeartbeat) {
+			        	logger("DBG", "Send buffered heartbeat message");
+					ivHeartbeat = 0;
+					writeIViewerHeartbeat();
+				}
+			} else {
+				ivInCommand = 1;
+			}
+			
 			bStr = strtok(NULL,",");
 		}
 	}
@@ -979,3 +1019,12 @@ void freeBtAddress()
 	if (btAddress) free(btAddress); 
 	btAddress = NULL;
 }
+
+void sendIViewerHeartbeat()
+{
+	if (ivInCommand) {
+		ivHeartbeat = 1;
+	} else {
+		writeIViewerHeartbeat();
+	}
+}
