@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009,2010 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -996,7 +996,7 @@ static int storeTimerCmd(cmdItem* ci, char *cmd)
         int cmdIsOk = EXIT_OK;
 	if (regexec(regex, cmd, no_sub, result, 0) == 0) {	// Match it
 
-		if (result[1].rm_so >= 0 &&	// Timer(key,0,5)
+		if (result[1].rm_so >= 0 &&	// Timer(key,1,5)
 		    result[2].rm_so >  0 &&
 		    result[3].rm_so >  0) {
                         
@@ -1013,16 +1013,22 @@ static int storeTimerCmd(cmdItem* ci, char *cmd)
                 	ci->tdata = (timerData*) calloc(1, sizeof(timerData));
                         
                 	char iBuff[8];
+			memset(iBuff,0,8);
                 	int l = result[2].rm_eo - result[2].rm_so;
                 	if (l > 7) l = 7;
                 	strncpy(iBuff,cmd + result[2].rm_so,l);
                 	ci->tdata->timeout = atoi(iBuff);
+			
+			//printf("storeTimerCmd():%d %s %s %d\n",l,ci->descr,iBuff,ci->tdata->timeout); 
                 
                 	l = result[3].rm_eo - result[3].rm_so;
                 	if (l > 7) l = 7;
-                	strncpy(iBuff,cmd + result[3].rm_so,7);
+ 			memset(iBuff,0,8);
+               		strncpy(iBuff,cmd + result[3].rm_so,l);
                 	ci->tdata->times = atoi(iBuff);
-                
+			
+ 			//printf("storeTimerCmd():%d %s %s %d\n",l,ci->descr,iBuff,ci->tdata->times); 
+               
                 } else if (result[4].rm_so >= 0 && 	// Timer(key,cancel|pause|continue)
                            result[5].rm_so >= 0) {  
 
@@ -1039,10 +1045,6 @@ static int storeTimerCmd(cmdItem* ci, char *cmd)
                 }
 	} else {
         	printf("storeTimerCmd(): command %s is formed incorrectly (2)\n", cmd);
-                
-                // info for next few releases
-        	printf("Since v4.1 syntax of Timer() command was changed. Read documentation for details.\n");
-                
                 cmdIsOk = EXIT_NOK;
         }
 
